@@ -17,7 +17,6 @@ The model cards toolkit (MCT) provides a set of utilities to help users
 generate Model Cards from trained models within ML pipelines.
 """
 
-import json
 import os
 import pkgutil
 import tempfile
@@ -27,9 +26,9 @@ from absl import logging
 import jinja2
 
 from model_card_toolkit.model_card import ModelCard
+from model_card_toolkit.proto import model_card_pb2
 from model_card_toolkit.utils import graphics
 from model_card_toolkit.utils import tfx_util
-from model_card_toolkit.utils import validation
 
 import ml_metadata as mlmd
 
@@ -39,8 +38,9 @@ _UI_TEMPLATES = (
     'template/md/default_template.md.jinja',
 )
 _DEFAULT_UI_TEMPLATE_FILE = os.path.join('html', 'default_template.html.jinja')
+
 # Constants about Model Cards Toolkit Assets (MCTA).
-_MCTA_JSON_FILE = 'data/model_card.json'
+_MCTA_PROTO_FILE = 'data/model_card.proto'
 _MCTA_TEMPLATE_DIR = 'template'
 _MCTA_RESOURCE_DIR = 'resources/plots'
 # Constants about the final generated model cards.
@@ -79,14 +79,15 @@ class ModelCardToolkit():
   model_card = mct.scaffold_assets()
   model_card.model_details.name = 'My Model'
 
-  # Write the model card data to a JSON file
-  mct.update_model_card_json(model_card)
+  # Write the model card data to a data file
+  mct.update_model_card(model_card)
 
   # Return the model card document as an HTML page
   html = mct.export_format()
   ```
   """
 
+  # TODO(b/188707257): combine mlmd_store and model_uri args
   def __init__(self,
                output_dir: Optional[Text] = None,
                mlmd_store: Optional[mlmd.MetadataStore] = None,
@@ -94,21 +95,23 @@ class ModelCardToolkit():
     """Initializes the ModelCardToolkit.
 
     Args:
-      output_dir: The MCT assets path where the json file and templates are
+      output_dir: The MCT assets path where the data files and templates are
         written to. If not given, a temp directory is used.
       mlmd_store: A ml-metadata MetadataStore to retrieve metadata and lineage
         information about the model stored at `model_uri`. If given, a set of
         model card properties can be auto-populated from the `mlmd_store`.
-      model_uri: The path to the trained model to generate model cards.
+      model_uri: The path to the trained model to generate model cards. Ignored
+        if mlmd_store is not used.
 
     Raises:
       ValueError: If `mlmd_store` is given and the `model_uri` cannot be
         resolved as a model artifact in the metadata store.
     """
     self.output_dir = output_dir or tempfile.mkdtemp()
-    self._mcta_json_file = os.path.join(self.output_dir, _MCTA_JSON_FILE)
+    self._mcta_proto_file = os.path.join(self.output_dir, _MCTA_PROTO_FILE)
     self._mcta_template_dir = os.path.join(self.output_dir, _MCTA_TEMPLATE_DIR)
     self._model_cards_dir = os.path.join(self.output_dir, _MODEL_CARDS_DIR)
+
     self._store = mlmd_store
     if self._store:
       if not model_uri:
@@ -121,6 +124,11 @@ class ModelCardToolkit():
             '%d artifacts are found with the `model_uri`="%s". '
             'The last one is used.', len(models), model_uri)
       self._artifact_with_model_uri = models[-1]
+    elif model_uri:
+      logging.info('model_uri ignored when mlmd_store is not present.')
+
+  def _jinja_loader(self, template_dir: Text):
+    return jinja2.FileSystemLoader(template_dir)
 
   def _write_file(self, path: Text, content: Text) -> None:
     """Write content to the path."""
@@ -128,28 +136,27 @@ class ModelCardToolkit():
     with open(path, 'w+') as f:
       f.write(content)
 
-  def _read_file(self, path: Text) -> Text:
-    """Read content from a path."""
-    with open(path, 'r') as f:
-      return f.read()
+  def _write_proto_file(self, path: Text, model_card: ModelCard) -> None:
+    """Write serialized model card proto to the path."""
+    os.makedirs(os.path.dirname(path), exist_ok=True)
+    with open(path, 'wb') as f:
+      f.write(model_card.to_proto().SerializeToString())
 
-  def scaffold_assets(self) -> ModelCard:
-    """Generates the model cards tookit assets.
+  def _read_proto_file(self, path: Text) -> ModelCard:
+    """Read serialized model card proto from the path."""
+    model_card_proto = model_card_pb2.ModelCard()
+    with open(path, 'rb') as f:
+      model_card_proto.ParseFromString(f.read())
+    return ModelCard().copy_from_proto(model_card_proto)
 
-    Model cards assets include the model card json file and customizable model
-    card UI templates.
+  def _scaffold_model_card(self) -> ModelCard:
+    """Generates the model card during scaffold_assets phase.
 
-    An assets directory is created if one does not already exist.
-
-    If the MCT is initialized with a `mlmd_store`, it further auto-populates
-    the model cards properties as well as generating related plots such as model
-    performance and data distributions.
+    It includes the implementation details for auto-populated ModelCard fields
+    given the specialization of the ModelCardToolkit.
 
     Returns:
       A ModelCard representing the given model.
-
-    Raises:
-      FileNotFoundError: if it failed to copy the UI template files.
     """
     model_card = ModelCard()
     if self._store:
@@ -169,10 +176,31 @@ class ModelCardToolkit():
         train_stats = tfx_util.read_stats_proto(stats_artifact.uri, 'train')
         eval_stats = tfx_util.read_stats_proto(stats_artifact.uri, 'eval')
         graphics.annotate_dataset_feature_statistics_plots(
-            model_card, train_stats, eval_stats)
+            model_card, [train_stats, eval_stats])
+    return model_card
 
-    # Write JSON file.
-    self._write_file(self._mcta_json_file, model_card.to_json())
+  def scaffold_assets(self) -> ModelCard:
+    """Generates the model cards tookit assets.
+
+    Model cards assets include the model card data files and customizable model
+    card UI templates.
+
+    An assets directory is created if one does not already exist.
+
+    If the MCT is initialized with a `mlmd_store`, it further auto-populates
+    the model cards properties as well as generating related plots such as model
+    performance and data distributions.
+
+    Returns:
+      A ModelCard representing the given model.
+
+    Raises:
+      FileNotFoundError: if it failed to copy the UI template files.
+    """
+    model_card = self._scaffold_model_card()
+
+    # Write Proto file.
+    self._write_proto_file(self._mcta_proto_file, model_card)
 
     # Write UI template files.
     for template_path in _UI_TEMPLATES:
@@ -186,10 +214,7 @@ class ModelCardToolkit():
     return model_card
 
   def update_model_card_json(self, model_card: ModelCard) -> None:
-    """Validates the model card and updates the JSON file in MCT assets.
-
-    If model_card.schema_version is not provided, it will assign the latest
-    schema version to the `model_card`, and validate it.
+    """Updates the Proto file in the MCT assets directory.
 
     Args:
       model_card: The updated model card that users want to write back.
@@ -197,18 +222,32 @@ class ModelCardToolkit():
     Raises:
        Error: when the given model_card is invalid w.r.t. the schema.
     """
-    if not model_card.schema_version:
-      model_card.schema_version = validation.get_latest_schema_version()
-    validation.validate_json_schema(model_card.to_dict(),
-                                    model_card.schema_version)
-    self._write_file(self._mcta_json_file, model_card.to_json())
+    logging.warning(
+        'update_model_card_json() will be deprecated in the next release. '
+        'Call update_model_card() and write to proto instead.')
+    self.update_model_card(model_card)
+
+  def update_model_card(self, model_card: ModelCard) -> None:
+    """Updates the Proto file in the MCT assets directory.
+
+    Args:
+      model_card: The updated model card that users want to write back.
+
+    Raises:
+       Error: when the given model_card is invalid w.r.t. the schema.
+    """
+    self._write_proto_file(self._mcta_proto_file, model_card)
 
   def export_format(self,
-                    template_path: Text = None,
+                    model_card: Optional[ModelCard] = None,
+                    template_path: Optional[Text] = None,
                     output_file=_DEFAULT_MODEL_CARD_FILE_NAME) -> Text:
     """Generates a model card based on the MCT assets.
 
     Args:
+      model_card: The ModelCard object, generated from `scaffold_assets()`. If
+        not provided, it will be read from the ModelCard proto file in the
+        assets directory.
       template_path: The file path of the UI template. If not provided, the
         default UI template will be used.
       output_file: The file name of the generated model card. If not provided,
@@ -216,31 +255,41 @@ class ModelCardToolkit():
         then it will be overwritten.
 
     Returns:
-      The model card UI.
+      The model card file content.
+
+    Raises:
+      MCTError: If `export_format` is called before `scaffold_assets` has
+        generated model card assets.
     """
     if not template_path:
       template_path = os.path.join(self._mcta_template_dir,
                                    _DEFAULT_UI_TEMPLATE_FILE)
-
     template_dir = os.path.dirname(template_path)
     template_file = os.path.basename(template_path)
 
-    # Read JSON file.
-    model_card = json.loads(self._read_file(self._mcta_json_file))
+    # If model_card is passed in, write to Proto file.
+    if model_card:
+      self.update_model_card(model_card)
+    # If model_card is not passed in, read from Proto file.
+    elif os.path.exists(self._mcta_proto_file):
+      model_card = self._read_proto_file(self._mcta_proto_file)
+    # If model card proto never created, raise exception.
+    else:
+      raise ValueError(
+          'scaffold_assets() must be called before export_format().')
 
     # Generate Model Card.
     jinja_env = jinja2.Environment(
-        loader=jinja2.FileSystemLoader(template_dir),
+        loader=self._jinja_loader(template_dir),
         autoescape=True,
         auto_reload=True,
         cache_size=0)
     template = jinja_env.get_template(template_file)
-    # TODO(b/154990170) Think about how to adjust the img inside template.
     model_card_file_content = template.render(
-        model_details=model_card['model_details'],
-        model_parameters=model_card.get('model_parameters', {}),
-        quantitative_analysis=model_card.get('quantitative_analysis', {}),
-        considerations=model_card.get('considerations', {}))
+        model_details=model_card.model_details,
+        model_parameters=model_card.model_parameters,
+        quantitative_analysis=model_card.quantitative_analysis,
+        considerations=model_card.considerations)
 
     # Write the model card file.
     mode_card_file_path = os.path.join(self._model_cards_dir, output_file)
